import geo_parameters as gp


def test_get_wave():
    assert gp.shortget("hs") == gp.wave.Hs
    assert gp.get("significant_wave_height") is None
    assert gp.get("sea_surface_wave_significant_height") == gp.wave.Hs
    assert gp.get("significant_height_of_wind_and_swell_waves") == gp.wave.Hs


def test_get_wind():
    assert gp.shortget("x_wind") == gp.wind.XWind
    assert gp.get("xwind") is None
    assert gp.get("eastward_wind_component") is None
    assert gp.get("x_wind") == gp.wind.XWind
    assert gp.get("grid_eastward_wind") == gp.wind.XWind


def test_get_iteration_short_name():
    params = gp.dict_of_parameters(short=True)
    for key, value in params.items():
        assert gp.shortget(key) == value


def test_get_iteration_standard_name():
    params = gp.dict_of_parameters()
    for key, value in params.items():
        assert gp.get(key) == value


def test_get_iteration_alias():
    params = gp.dict_of_parameters(alias=True)
    for key, value in params.items():
        assert gp.get(key) == value



import geo_parameters as gp


def test_wavenumber():
    km = gp.wave.Km()
    assert km.name == "km"
    assert (
        km.standard_name()
        == "sea_surface_wave_mean_wavenumber_from_variance_spectral_density_first_wavenumber_moment"
    )
    km = gp.wave.Km("mean_k")
    assert km.name == "mean_k"
    assert (
        km.standard_name()
        == "sea_surface_wave_mean_wavenumber_from_variance_spectral_density_first_wavenumber_moment"
    )


def test_wavelength():
    lm = gp.wave.Lm_10()
    assert lm.name == "lm_10"
    assert (
        lm.standard_name()
        == "sea_surface_wave_mean_wavelength_from_variance_spectral_density_inverse_wavenumber_moment"
    )
    lm = gp.wave.Lm_10("mean_l")
    assert lm.name == "mean_l"
    assert (
        lm.standard_name()
        == "sea_surface_wave_mean_wavelength_from_variance_spectral_density_inverse_wavenumber_moment"
    )



from geo_parameters.metaparameter import MetaParameter
from geo_parameters.ureg import ureg
from .relationships import RELATIONSHIPS
from typing import Optional


class OceanParameter(MetaParameter):
    @classmethod
    def my_family(cls, param_type: Optional[str] = None) -> dict:
        """Returns the dictonary containing the parameters where cls is in.
        Use .my_family('direction') to get the parameter isntead of a dict"""
        if param_type is not None:
            assert param_type in [
                "magnitude",
                "direction",
                "opposite_direction",
                "x",
                "y",
                "period",
                "frequency",
            ]

        for rel in RELATIONSHIPS:
            for param in rel.values():
                if type(cls()).__name__ == param:
                    # This needs to be done to circument circular imports etc
                    eval_rel = {}
                    for key, value in rel.items():
                        eval_rel[key] = eval(value)
                    if param_type is not None:
                        return eval_rel.get(param_type)
                    else:
                        return eval_rel

        return {}


class WaterDepth(OceanParameter):
    name = "depth"
    _long_name = "water_depth"
    _standard_name = "sea_floor_depth_below_sea_surface"
    _unit = ureg.m


class SeaLevel(OceanParameter):
    name = "eta"
    _long_name = "sea_surface_height"
    _standard_name = [
        "sea_surface_elevation",
        "sea_surface_elevation_anomaly",
        "sea_surface_height_above_geoid",
    ]
    _unit = ureg.m


class XCurrent(OceanParameter):
    name = "x_current"
    _long_name = "eastward_current_component"
    _standard_name = [
        "sea_water_x_velocity",
        "x_sea_water_velocity",
    ]
    _unit = ureg.m / ureg.s


class YCurrent(OceanParameter):
    name = "y_current"
    _long_name = "northward_current_component"
    _standard_name = [
        "sea_water_y_velocity",
        "y_sea_water_velocity",
    ]
    _unit = ureg.m / ureg.s


class Current(OceanParameter):
    name = "current"
    _long_name = "current_speed"
    _standard_name = "sea_water_speed"
    _unit = ureg.m / ureg.s


class CurrentDir(OceanParameter):
    name = "current_dir"
    _long_name = "current_direction"
    _standard_name = [
        "sea_water_velocity_to_direction",
        "sea_water_to_direction",
        "direction_of_sea_water_velocity",
    ]
    _unit = ureg.deg


class CurrentDirFrom(OceanParameter):
    name = "current_dir"
    _long_name = "current_direction"
    _standard_name = [
        "sea_water_velocity_from_direction",
        "sea_water_from_direction",
    ]
    _unit = ureg.deg


class IceFraction(OceanParameter):
    name = "ice_fraction"
    _long_name = "sea_ice_fraction"
    _standard_name = "sea_ice_area_fraction"
    _unit = ureg.percent


class IceThickness(OceanParameter):
    name = "ice_thickness"
    _long_name = "sea_ice_thickness"
    _standard_name = "sea_ice_thickness"
    _unit = ureg.m



import geo_parameters as gp


def test_no_family():

    assert gp.wave.Hs.my_family() == {}
    assert gp.wave.SprPSea().my_family() == {}


def test_wind_family():
    assert set(gp.wind.XWind().my_family().values()) == {
        gp.wind.XWind,
        gp.wind.YWind,
        gp.wind.Wind,
        gp.wind.WindDir,
        gp.wind.WindDirTo,
    }

    assert gp.wind.XWind().my_family().get("x") == gp.wind.XWind().my_family("x")

    assert set(gp.wind.XWind.my_family().values()) == {
        gp.wind.XWind,
        gp.wind.YWind,
        gp.wind.Wind,
        gp.wind.WindDir,
        gp.wind.WindDirTo,
    }
    assert (
        gp.wind.XWind.my_family().get("opposite_direction").is_same(gp.wind.WindDirTo)
    )


def test_stokes_family():
    assert set(gp.wave.XStokes().my_family().values()) == {
        gp.wave.XStokes,
        gp.wave.YStokes,
        gp.wave.Stokes,
        gp.wave.StokesDir,
        gp.wave.StokesDirFrom,
    }

    assert set(gp.wave.YStokes.my_family().values()) == {
        gp.wave.XStokes,
        gp.wave.YStokes,
        gp.wave.Stokes,
        gp.wave.StokesDir,
        gp.wave.StokesDirFrom,
    }

    assert (
        gp.wave.XStokes.my_family()
        .get("opposite_direction")
        .is_same(gp.wave.StokesDirFrom)
    )


def test_current_family():
    assert set(gp.ocean.XCurrent().my_family().values()) == {
        gp.ocean.XCurrent,
        gp.ocean.YCurrent,
        gp.ocean.Current,
        gp.ocean.CurrentDir,
        gp.ocean.CurrentDirFrom,
    }

    assert set(gp.ocean.YCurrent.my_family().values()) == {
        gp.ocean.XCurrent,
        gp.ocean.YCurrent,
        gp.ocean.Current,
        gp.ocean.CurrentDir,
        gp.ocean.CurrentDirFrom,
    }

    assert (
        gp.ocean.XCurrent.my_family()
        .get("opposite_direction")
        .is_same(gp.ocean.CurrentDirFrom)
    )


def test_periods():
    assert gp.wave.Tp.i_am() == "period"
    assert gp.wave.Tp.my_family().get("frequency").is_same(gp.wave.Fp)
    assert gp.wave.Tp.my_family().get("angular_frequency").is_same(gp.wave.Wp)



import geo_parameters as gp


def test_is_gpi():
    assert gp.is_gp_instance(gp.wave.Hs())
    assert gp.is_gp_instance(gp.wave.Hs("test"))
    assert not gp.is_gp_instance(gp.wave.Hs)
    assert not gp.is_gp_instance("test")


def test_is_gpc():
    assert not gp.is_gp_class(gp.wave.Hs())
    assert not gp.is_gp_class(gp.wave.Hs("test"))
    assert gp.is_gp_class(gp.wave.Hs)
    assert not gp.is_gp_class("test")


def test_is_gp():
    assert gp.is_gp(gp.wave.Hs())
    assert gp.is_gp(gp.wave.Hs("test"))
    assert gp.is_gp(gp.wave.Hs)
    assert not gp.is_gp("test")



from geo_parameters.metaparameter import MetaParameter
from geo_parameters.ureg import ureg
from typing import Optional
from .relationships import RELATIONSHIPS


class WindParameter(MetaParameter):
    @classmethod
    def my_family(cls, param_type: Optional[str] = None) -> dict:
        """Returns the dictonary containing the parameters where cls is in.
        Use .my_family('direction') to get the parameter isntead of a dict"""
        if param_type is not None:
            assert param_type in [
                "magnitude",
                "direction",
                "opposite_direction",
                "x",
                "y",
                "period",
                "frequency",
            ]

        for rel in RELATIONSHIPS:
            for param in rel.values():
                if type(cls()).__name__ == param:
                    # This needs to be done to circument circular imports etc
                    eval_rel = {}
                    for key, value in rel.items():
                        eval_rel[key] = eval(value)
                    if param_type is not None:
                        return eval_rel.get(param_type)
                    else:
                        return eval_rel

        return {}


class XWind(WindParameter):
    name = "x_wind"
    _long_name = "x_wind_component"
    _standard_name = [
        "x_wind",
        "grid_eastward_wind",
    ]
    _unit = ureg.m / ureg.s


class YWind(WindParameter):
    name = "y_wind"
    _long_name = "y_wind_component"
    _standard_name = [
        "y_wind",
        "grid_northward_wind",
    ]
    _unit = ureg.m / ureg.s


class Wind(WindParameter):
    name = "ff"
    _long_name = "wind_speed"
    _standard_name = "wind_speed"
    _unit = ureg.m / ureg.s


class WindDir(WindParameter):
    name = "dd"
    _long_name = "wind_direction"
    _standard_name = "wind_from_direction"
    _unit = ureg.deg


class WindDirTo(WindParameter):
    name = "dd"
    _long_name = "wind_direction"
    _standard_name = "wind_to_direction"
    _unit = ureg.deg


class XGust(WindParameter):
    name = "x_gust"
    _long_name = "x_gust_component"
    _standard_name = "x_wind_gust"
    _unit = ureg.m / ureg.s


class YGust(WindParameter):
    name = "y_gust"
    _long_name = "y_gust_component"
    _standard_name = "y_wind_gust"
    _unit = ureg.m / ureg.s


class Gust(WindParameter):
    name = "gust"
    _long_name = "wind_gust"
    _standard_name = "wind_speed_of_gust"
    _unit = ureg.m / ureg.s


class GustDir(WindParameter):
    name = "gust_dir"
    _long_name = "wind_gust_direction"
    _standard_name = "wind_gust_from_direction"
    _unit = ureg.deg


class GustDirTo(WindParameter):
    name = "gust_dir"
    _long_name = "wind_gust_direction"
    _standard_name = "wind_gust_to_direction"
    _unit = ureg.deg
    _cf = False



from geo_parameters.metaparameter import MetaParameter
from geo_parameters.ureg import ureg
from .relationships import RELATIONSHIPS
from typing import Optional


class GridParameter(MetaParameter):
    @classmethod
    def my_family(cls, param_type: Optional[str] = None) -> dict:
        """Returns the dictonary containing the parameters where cls is in.
        Use .my_family('direction') to get the parameter isntead of a dict"""
        if param_type is not None:
            assert param_type in [
                "magnitude",
                "direction",
                "opposite_direction",
                "x",
                "y",
                "period",
                "frequency",
            ]

        for rel in RELATIONSHIPS:
            for param in rel.values():
                if type(cls()).__name__ == param:
                    # This needs to be done to circument circular imports etc
                    eval_rel = {}
                    for key, value in rel.items():
                        eval_rel[key] = eval(value)
                    if param_type is not None:
                        return eval_rel.get(param_type)
                    else:
                        return eval_rel

        return {}


class Lon(GridParameter):
    name = "lon"
    _long_name = "longitude"
    _standard_name = "longitude"
    _unit = "degrees_east"


class Lat(GridParameter):
    name = "lat"
    _long_name = "latitude"
    _standard_name = "latitude"
    _unit = "degrees_north"


class X(GridParameter):
    name = "x"
    _long_name = "x_distance"
    _standard_name = "distance_in_x_direction"
    _unit = ureg.m
    _cf = False


class Y(GridParameter):
    name = "y"
    _long_name = "y_distance"
    _standard_name = "distance_in_y_direction"
    _unit = ureg.m
    _cf = False


class Inds(GridParameter):
    name = "inds"
    _long_name = "index_of_points"
    _standard_name = "index_of_geophysical_points"
    _unit = "-"
    _cf = False



"""This module keeps track of the relationships between the parameters"""

RELATIONSHIPS = [
    {
        "x": "XWind",
        "y": "YWind",
        "magnitude": "Wind",
        "direction": "WindDir",
        "opposite_direction": "WindDirTo",
    },
    {
        "x": "XGust",
        "y": "YGust",
        "magnitude": "Gust",
        "direction": "GustDir",
        "opposite_direction": "GustDirTo",
    },
    {
        "x": "XCurrent",
        "y": "YCurrent",
        "magnitude": "Current",
        "direction": "CurrentDir",
        "opposite_direction": "CurrentDirFrom",
    },
    {
        "x": "XStokes",
        "y": "YStokes",
        "magnitude": "Stokes",
        "direction": "StokesDir",
        "opposite_direction": "StokesDirFrom",
    },
    {"period": "Tp", "frequency": "Fp", "angular_frequency": "Wp"},
    {"period": "TpSwell", "frequency": "FpSwell"},
    {"period": "TpSwell1", "frequency": "FpSwell1"},
    {"period": "TpSwell2", "frequency": "FpSwell2"},
    {"period": "TpSwell3", "frequency": "FpSwell3"},
    {"period": "Tm01", "frequency": "Fm", "angular_frequency": "Wm"},
    {"period": "TmSwell", "frequency": "FmSwell"},
    {"period": "TmSwell1", "frequency": "FmSwell1"},
    {"period": "TmSwell2", "frequency": "FmSwell2"},
    {"period": "TmSwell3", "frequency": "FmSwell3"},
]
